<?php

// 数据库连接的抽象层
abstract class DB_Connector {
	
	protected static
		$_register = array();

	static public function connect($anyKey) {
		// 假设传入的$anyKey指定要使用MySQL进行连接
		// 这中间的一些判断这里就忽略了
		if (!isset(self::$_register[$anyKey])) {
			self::$_register[$anyKey] = new DB_Connector_MySQL();
		}
		return self::$_register[$anyKey];
	}

	static public function disconnect($anyKey) {
		self::connect($anyKey)->disconnect();
	}

	static public function handleDisconnect($anyKey) {
		self::connect($anyKey)->handleDisconnect();
	}
}

// 数据库连接的驱动层
class DB_Connector_MySQL {
	
	protected
		$_connector = null,
		$_isHandleDisconnect = false;

	public function __construct() {
		// 执行具体的连接
		$this->_connector = new MySQLDriver();
	}
	
	public function __destruct() {
		if (!$this->_isHandleDisconnect)
			$this->disconnect();
	}

	public function disconnect() {
		$this->_connector = null;
	}

	public function handleDisconnect() {
		$this->_isHandleDisconnect = true;
	}
}

// Session的实现层
// Any_ActiveRecord是Model的抽象层，这里就不实现了
class Session extends Any_ActiveRecord {
	
	protected static
		$_connectorKey = 'Any',
		$_currSess = null;

	// 标准实现
	static public function open() {
		// 一旦将Session处理转移给DB层面去控制
		// 就意味着数据库连接的释放，也必须转交给这个Session模块来处理
		DB_Connector::handleDisconnect(self::$_connectorKey);
		// 其他启动配置，包括Session GC清理的基数等等
	}
	
	// 标准实现
	// 拿出Session
	static public function pick($sId) {
		self::$_currSess = self::find_by_session_id($sId);
		if (!self::$_currSess->isEmpty())
			return self::$_currSess->value;
		return false;
	}

	// 标准实现
	static public function dump($sId, $val) {
		// 新访客
		if (self::$_currSess->isEmpty())
			self::$_currSess->session_id == $sId;
		self::$_currSess->value = $val;
		self::$_currSess->save();
	}

	// 标准实现
	static public function destroy($sId) {
		
	}

	// 标准实现
	static public function gc() {
		
	}

	// 标准实现
	static public function close() {
		// 一切OK，再由Session Close的时候，释放数据库连接
		DB_Connector::disconnect(self::$_connectorKey);
		self::$_currSess = null;
	}
}

?>